=== modified file 'libdrizzle/query.c'
--- libdrizzle/query.c	2010-09-28 07:47:48 +0000
+++ libdrizzle/query.c	2010-10-17 16:04:42 +0000
@@ -325,29 +325,45 @@
 size_t drizzle_escape_string(char *to, const char *from, size_t from_size)
 {
   size_t to_size= 0;
+  char newchar;
 
   while (from_size > 0)
   {
+    newchar= 0;
     /* All multi-byte UTF8 characters have the high bit set for all bytes. */
     if (!(*from & 0x80))
     {
       switch (*from)
       {
       case 0:
+        newchar= '0';
+        break;
       case '\n':
+        newchar= 'n';
+        break;
       case '\r':
+        newchar= 'r';
+        break;
+      case '\032':
+        newchar= 'Z';
+        break;
       case '\\':
       case '\'':
       case '"':
-      case '\032':
         *to++= '\\';
         to_size++;
       default:
         break;
       }
     }
-
-    *to++= *from++;
+    if (newchar != 0)
+    {
+      *to++= '\\';
+      *to++= newchar;
+      to_size++;
+    }
+    else 
+      *to++= *from++;
     from_size--;
     to_size++;
   }

=== modified file 'unittests/include.am'
--- unittests/include.am	2010-09-21 17:31:14 +0000
+++ unittests/include.am	2010-10-17 16:04:42 +0000
@@ -27,6 +27,7 @@
 			      unittests/date_test.cc \
 			      unittests/date_time_test.cc \
 			      unittests/generators.cc \
+                              unittests/libdrizzle_test.cc \
 			      unittests/main.cc \
 			      unittests/micro_timestamp_test.cc \
 			      unittests/nano_timestamp_test.cc \
@@ -44,4 +45,5 @@
 
 unittests_unittests_LDADD= \
   $(filter-out drizzled/main.$(OBJEXT), ${am_drizzled_drizzled_OBJECTS}) \
-  ${drizzled_drizzled_LDADD} ${LTLIBGTEST} ${BOOST_LIBS}
+  ${drizzled_drizzled_LDADD} ${LTLIBGTEST} ${BOOST_LIBS} \
+  libdrizzle/libdrizzle.la

=== added file 'unittests/libdrizzle_test.cc'
--- unittests/libdrizzle_test.cc	1970-01-01 00:00:00 +0000
+++ unittests/libdrizzle_test.cc	2010-10-17 16:04:42 +0000
@@ -0,0 +1,59 @@
+/* -*- mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2010 Andrew Hutchings
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#include "config.h"
+
+#include <gtest/gtest.h>
+#include <libdrizzle/drizzle_client.h>
+#include <libdrizzle/drizzle_server.h>
+
+TEST(libdrizzle, drizzle_escape_string)
+{
+  const char* orig= "hello \"world\"\n";
+  char out[255];
+  size_t out_len;
+
+  out_len= drizzle_escape_string(out, orig, strlen(orig));
+
+  EXPECT_EQ(17, out_len);
+  ASSERT_STREQ("hello \\\"world\\\"\\n", out);
+}
+
+TEST(libdrizzle, drizzle_hex_string)
+{
+  const unsigned char orig[5]= {0x34, 0x26, 0x80, 0x99, 0xFF};
+  char out[255];
+  size_t out_len;
+
+  out_len= drizzle_hex_string(out, (char*) orig, 5);
+
+  EXPECT_EQ(10, out_len);
+  ASSERT_STREQ("34268099FF", out);
+}
+
+TEST(libdrizzle, drizzle_mysql_password_hash)
+{
+  const char* orig= "test password";
+  char out[255];
+
+  drizzle_mysql_password_hash(out, orig, strlen(orig));
+
+  ASSERT_STREQ("3B942720DACACBBA7E3838AF03C5B6B5A6DFE0AB", out);
+}

